@@ -22,7 +22,23 @@ else
     FORK=`git merge-base $MAIN HEAD || true`
     if test x$FORK = x
     then
-        : Failing.  HEAD does not appear to have a common fork point with $MAIN.
+        : Failing.  HEAD does not appear to have a common fork point
+        : with $MAIN.
+        :
+        : If HEAD and $MAIN have a common fork point, then you
+        : probably did a shallow clone.
+        :
+        git rev-parse --is-shallow-repository
+        :
+        : If so, go to:
+        :
+        :   Project / Settings / CI/CD / General pipelines / Git strategy
+        :
+        : And make sure "git shallow clone" is set to "0".  This causes
+        : gitlab to fetch all of the history.
+        :
+        : Alternatively, you can use the GIT_DEPTH variable to set this on
+        : a per-job basis.
         exit 1
     else
         : MR needs to be merged or rebased.  Fork point is $FORK.
