@@ -6,8 +6,11 @@ use std::{
         PathBuf,
     }
 };
+
 use anyhow::{anyhow, Context, Result};
-use clap::Parser;
+
+use clap::FromArgMatches;
+
 use once_cell::sync::OnceCell;
 
 use sequoia_openpgp::{
@@ -27,6 +30,8 @@ mod macros;
 
 mod cli;
 use cli::CertArg;
+use cli::PolicyFileArg;
+use cli::paths::StateDirectory;
 
 mod commands;
 mod output;
@@ -84,7 +89,7 @@ impl CertArg {
                     })
             }
             Err(kh) => {
-                let certs = config.cert_store()?.lookup_by_key(&kh)?;
+                let certs = config.cert_store()?.lookup_by_cert_or_subkey(&kh)?;
 
                 let cert = match certs.len() {
                     0 => return Err(anyhow!("Key {} not found", kh)),
@@ -102,14 +107,33 @@ impl CertArg {
 }
 
 pub struct Config<'a> {
-    policy_file: Option<&'a Path>,
     output_format: output::Format,
-    no_cert_store: bool,
-    cert_store: Option<PathBuf>,
-    cert_store_instance: OnceCell<CertStore<'a>>,
+    home: Option<sequoia_directories::Home>,
+    cert_store_path: Option<StateDirectory>,
+    cert_store: OnceCell<CertStore<'a>>,
 }
 
 impl<'a> Config<'a> {
+    /// Returns the cert store's base directory, if it is enabled.
+    fn cert_store_base(&self) -> Option<PathBuf> {
+        let default = || if let Ok(path) = std::env::var("PGP_CERT_D") {
+            Some(PathBuf::from(path))
+        } else {
+            self.home.as_ref()
+                .map(|h| h.data_dir(sequoia_directories::Component::CertD))
+        };
+
+        if let Some(state) = self.cert_store_path.as_ref() {
+            match state {
+                StateDirectory::Absolute(p) => Some(p.clone()),
+                StateDirectory::Default => default(),
+                StateDirectory::None => None,
+            }
+        } else {
+            default()
+        }
+    }
+
     // Returns the cert store.
     //
     // If it is not yet open, opens it.
@@ -117,44 +141,78 @@ impl<'a> Config<'a> {
     // If it does not exist, issues a warning and returns an empty
     // cert store.
     fn cert_store(&self) -> Result<&CertStore<'a>> {
-        self.cert_store_instance.get_or_try_init(|| {
-            if self.no_cert_store {
-                Ok(CertStore::empty())
-            } else {
-                let path = self.cert_store.clone()
-                    .unwrap_or_else(|| cli::cert_store_base().into());
+        if let Some(cert_store) = self.cert_store.get() {
+            // The cert store is already initialized, return it.
+            return Ok(cert_store);
+        }
 
-                match path.metadata() {
-                    Ok(metadata) => {
-                        if metadata.is_dir() {
-                            CertStore::open(&path)
-                        } else {
-                            Err(anyhow::anyhow!(
-                                "Not a certificate directory"))
-                        }
-                    }
-                    Err(err) => {
-                        if err.kind() == std::io::ErrorKind::NotFound {
-                            eprintln!("Warning: no certificate \
-                                       store found at {}",
-                                      path.display());
-                            Ok(CertStore::empty())
-                        } else {
-                            Err(err.into())
-                        }
-                    }
-                }
-                .with_context(|| {
-                    format!("While opening the certificate store at {:?}",
-                            &path)
-                })
+        let create_dirs = |path: &Path| -> Result<()> {
+            use std::fs::DirBuilder;
+
+            let mut b = DirBuilder::new();
+            b.recursive(true);
+
+            // Create the parent with the normal umask.
+            if let Some(parent) = path.parent() {
+                // Note: since recursive is turned on, it is not an
+                // error if the directory exists, which is exactly
+                // what we want.
+                b.create(parent)
+                    .with_context(|| {
+                        format!("Creating the directory {:?}", parent)
+                    })?;
             }
-        })
+
+            // Create path with more restrictive permissions.
+            platform!{
+                unix => {
+                    use std::os::unix::fs::DirBuilderExt;
+                    b.mode(0o700);
+                },
+                windows => {
+                },
+            }
+
+            b.create(path)
+                .with_context(|| {
+                    format!("Creating the directory {:?}", path)
+                })?;
+
+            Ok(())
+        };
+
+        // We need to initialize the cert store.
+
+        // Open the cert-d.
+
+        let path = self.cert_store_base()
+            .expect("just checked that it is configured");
+
+        let cert_store = create_dirs(&path)
+            .and_then(|_| CertStore::open(&path))
+            .with_context(|| {
+                format!("While opening the certificate store at {:?}",
+                        &path)
+            })?;
+
+        let _ = self.cert_store.set(cert_store);
+
+        Ok(self.cert_store.get().expect("just configured"))
     }
 
-    fn read_policy(&self) -> Result<Policy> {
-        if let Some(path) = self.policy_file {
-            Policy::read(path)
+    fn read_policy<'f, F>(&self, policy_file: F) -> Result<Policy>
+    where
+        F: Into<Option<&'f PolicyFileArg>>,
+    {
+        let policy_file = policy_file.into();
+        let policy_file = if let Some(policy_file) = policy_file.as_ref() {
+            policy_file.path.as_ref()
+        } else {
+            None
+        };
+
+        if let Some(path) = policy_file {
+            Policy::read_file(path)
                 .with_context(|| {
                     format!("Reading specified policy file: {}",
                             path.display())
@@ -167,8 +225,43 @@ impl<'a> Config<'a> {
         }
     }
 
-    fn write_policy(&self, p: &Policy) -> Result<()> {
-        if let Some(path) = self.policy_file {
+    fn read_policy_or_default<'f, F>(&self, policy_file: F) -> Result<Policy>
+    where
+        F: Into<Option<&'f PolicyFileArg>>,
+    {
+        let policy_file = policy_file.into();
+        let policy_file = if let Some(policy_file) = policy_file.as_ref() {
+            policy_file.path.as_ref()
+        } else {
+            None
+        };
+
+        if let Some(path) = policy_file {
+            Policy::read_file_or_default(path)
+                .with_context(|| {
+                    format!("Reading specified policy file: {}",
+                            path.display())
+                })
+        } else {
+            Policy::read_from_working_dir()
+                .with_context(|| {
+                    format!("Reading default policy file")
+                })
+        }
+    }
+
+    fn write_policy<'f, F>(&self, p: &Policy, policy_file: F) -> Result<()>
+    where
+        F: Into<Option<&'f PolicyFileArg>>,
+    {
+        let policy_file = policy_file.into();
+        let policy_file = if let Some(policy_file) = policy_file.as_ref() {
+            policy_file.path.as_ref()
+        } else {
+            None
+        };
+
+        if let Some(path) = policy_file {
             p.write(path)
                 .with_context(|| {
                     format!("Updating the specified policy file: {}",
@@ -198,14 +291,96 @@ fn git_repo() -> Result<git2::Repository> {
 
 fn main() -> anyhow::Result<()> {
     let policy = openpgp::policy::StandardPolicy::new(); // XXX
-    let cli = cli::Cli::parse();
+
+    let mut cli = cli::build(true);
+    let matches = cli.clone().try_get_matches();
+    let mut matches = match matches {
+        Ok(matches) => matches,
+        Err(mut err) => {
+            // Warning: hack ahead!
+            //
+            // We want to hide global options in the help output for
+            // subcommands, and we want to include values from the
+            // configuration file in the help output.
+            //
+            // If we are showing the help output, we only want to
+            // display the global options at the top-level; for
+            // subcommands we hide the global options to not overwhelm
+            // the user.
+            //
+            // Ideally, clap would provide a mechanism to only show
+            // the help output for global options at the level they
+            // are defined at.  That's not the case.
+            //
+            // We can use `err` to figure out if we are showing the
+            // help output, but it doesn't tell us what subcommand we
+            // are showing the help for.  Instead (and here's the
+            // hack!), we compare the output.  If it is the output for
+            // the top-level `--help` or `-h`, then we are showing the
+            // help for the top-level.  If not, then we are showing
+            // the help for a subcommand.  In the former case, we
+            // unhide the global options.
+            use clap::error::ErrorKind;
+            if err.kind() == ErrorKind::DisplayHelp
+                || err.kind() == ErrorKind::DisplayHelpOnMissingArgumentOrSubcommand
+            {
+                let output = err.render();
+                let output = if output == cli.render_long_help() {
+                    Some(cli::build(false).render_long_help())
+                } else if output == cli.render_help() {
+                    Some(cli::build(false).render_help())
+                } else {
+                    // Redo the parse so that the help message will
+                    // include any augmentations.
+                    err = cli::build(true).try_get_matches().unwrap_err();
+                    None
+                };
+
+                if let Some(output) = output {
+                    if err.use_stderr() {
+                        eprint!("{}", output);
+                    } else {
+                        print!("{}", output);
+                    }
+                    std::process::exit(err.exit_code());
+                }
+            }
+
+            // Print the error message.
+            err.print()?;
+
+            // Then, figure out if this is a usage message, and
+            // extract the usage string.
+            if let Some(usage) = err.context()
+                .find_map(|(kind, value)|
+                          (kind == clap::error::ContextKind::Usage)
+                          .then_some(value))
+            {
+                print_examples(&cli, usage)?;
+            }
+
+            // Finally, exit with an error code.
+            std::process::exit(err.exit_code());
+        }
+    };
+    let cli = cli::Cli::from_arg_matches_mut(&mut matches)?;
+
+    let home = match &cli.home {
+        Some(StateDirectory::Absolute(p)) =>
+            Some(sequoia_directories::Home::new(p.clone())?),
+        None | Some(StateDirectory::Default) =>
+            Some(sequoia_directories::Home::default()
+                 .ok_or(anyhow::anyhow!("no default SEQUOIA_HOME \
+                                         on this platform"))?
+                 .clone()),
+        Some(StateDirectory::None) => None,
+    };
 
     let config = Config {
-        policy_file: cli.policy_file.as_deref(),
         output_format: cli.output_format.parse()?,
-        no_cert_store: cli.no_cert_store,
-        cert_store: cli.cert_store,
-        cert_store_instance: Default::default(),
+        home,
+        cert_store_path: cli.cert_store,
+        cert_store: Default::default(),
     };
 
     let commit_by_symbolic_name = |git: &git2::Repository, name: &str,
@@ -265,19 +440,24 @@ fn main() -> anyhow::Result<()> {
         let config = git.config()?
             .open_level(git2::ConfigLevel::Local)?
             .snapshot()?;
-        let trust_root = match config.get_str("sequoia.trust-root") {
+        let trust_root = match config.get_str("sequoia.trustRoot") {
             Ok(trust_root) => trust_root,
             Err(err) => {
                 if err.code() == git2::ErrorCode::NotFound {
                     eprintln!("Warning: no trust root specified.  Either \
                                pass the '--trust-root' option, or set \
-                               the 'sequoia.trust-root' configuration \
+                               the 'sequoia.trustRoot' configuration \
                                key in your repository's local git config \
                                to reference a commit.");
+                    if config.get_str("sequoia.trust-root").is_ok() {
+                        eprintln!("Warning: you seem to have set the \
+                                   `sequoia.trust-root` key.  This key \
+                                   has been deprecated.");
+                    }
                 }
 
                 return Err(anyhow::Error::from(err).context(
-                    "Reading 'sequoia.trust-root' from the repository's \
+                    "Reading 'sequoia.trustRoot' from the repository's \
                      git config."));
             }
         };
@@ -295,12 +475,15 @@ fn main() -> anyhow::Result<()> {
         }
 
         cli::Subcommand::Log {
+            policy_file,
             trust_root,
             keep_going,
             prune_certs,
             commit_range,
         } => {
-            if prune_certs && commit_range.is_some() && cli.policy_file.is_none() {
+            if prune_certs && commit_range.is_some()
+                && policy_file.path.is_none()
+            {
                 return Err(anyhow!("--prune-certs can only modify \
                                     HEAD or a shadow policy"));
             }
@@ -308,7 +491,7 @@ fn main() -> anyhow::Result<()> {
             let git = git_repo()?;
             let trust_root = lookup_trust_root(&git, trust_root.as_deref())?;
 
-            let shadow_p = if let Some(s) = &cli.policy_file {
+            let shadow_p = if let Some(s) = &policy_file.path {
                 Some(std::fs::read(s)?)
             } else {
                 None
@@ -344,7 +527,8 @@ fn main() -> anyhow::Result<()> {
                            keep_going,
                            |oid, parent_oid, result| {
                                output::Commit::new(
-                                   &git, oid, parent_oid, &cli.policy_file, result)?
+                                   &git, oid, parent_oid,
+                                   &policy_file.path, result)?
                                    .describe(&mut io::stdout())?;
                                Ok(())
                            },
@@ -364,7 +548,8 @@ fn main() -> anyhow::Result<()> {
                                |oid, parent_oid, result| {
                                    seq.serialize_element(
                                        &output::Commit::new(
-                                           &git, oid, parent_oid, &cli.policy_file, result)?
+                                           &git, oid, parent_oid,
+                                           &policy_file.path, result)?
                                    ).map_err(anyhow::Error::from)?;
                                    Ok(())
                                },
@@ -376,9 +561,9 @@ fn main() -> anyhow::Result<()> {
             };
 
             if prune_certs {
-                let mut p = config.read_policy()?;
+                let mut p = config.read_policy(&policy_file)?;
 
-                for a in p.authorization.values_mut() {
+                for a in p.authorization_mut().values_mut() {
                     let certs =
                         a.certs()?
                         .map(|r| r.and_then(Cert::try_from))
@@ -389,7 +574,7 @@ fn main() -> anyhow::Result<()> {
                         // Keep all subkeys that made a signature, and
                         // those that are alive now.
                         |sk| {
-                            let fp = sk.fingerprint();
+                            let fp = sk.key().fingerprint();
                             vresults.signer_keys.contains(&fp)
                                 || {
                                     // Slightly awkward, because we
@@ -405,7 +590,7 @@ fn main() -> anyhow::Result<()> {
                         // Keep all user IDs that were primary user
                         // IDs when a signature was made, and the ones
                         // that are the primary userid now.
-                        |uid| vresults.primary_uids.contains(&uid)
+                        |uid| vresults.primary_uids.contains(uid.userid())
                             || {
                                 // Slightly awkward, because we
                                 // cannot use sk.with_policy.
@@ -419,7 +604,7 @@ fn main() -> anyhow::Result<()> {
                     )?;
                 }
 
-                config.write_policy(&p)?;
+                config.write_policy(&p, &policy_file)?;
             }
 
             let _ = cache.persist();
@@ -427,13 +612,14 @@ fn main() -> anyhow::Result<()> {
         },
 
         cli::Subcommand::Verify {
+            policy_file,
             trust_root,
             signature,
             archive,
         } => {
             let git = git_repo()?;
-            let policy = if let Some(s) = &cli.policy_file {
-                Policy::read(s)
+            let policy = if let Some(s) = policy_file.path.as_ref() {
+                Policy::read_file(s)
                     .with_context(|| {
                         format!("Reading specified policy file: {}",
                                 s.display())
@@ -472,6 +658,7 @@ fn main() -> anyhow::Result<()> {
         },
 
         cli::Subcommand::UpdateHook {
+            policy_file,
             trust_root,
             ref_name: _,
             old_object,
@@ -524,7 +711,7 @@ fn main() -> anyhow::Result<()> {
                    false,
                    |oid, parent_oid, result| {
                        output::Commit::new(
-                           &git, oid, parent_oid, &cli.policy_file, result)?
+                           &git, oid, parent_oid, &policy_file.path, result)?
                            .describe(&mut io::stdout())?;
                        Ok(())
                    },
@@ -537,3 +724,53 @@ fn main() -> anyhow::Result<()> {
     }
     Ok(())
 }
+
+/// Given a `clap::Command` and a usage string, try to augment the
+/// usage message with relevant examples.
+fn print_examples(cli: &clap::Command, usage: impl ToString) -> Result<()> {
+    // First, find the invoked subcommand.
+    let binary = "sq-git";
+    let usage = usage.to_string();
+    let prefix = usage.find(&format!("{} ", binary));
+    let subcommands = if let Some(i) = prefix {
+        usage[i + binary.len() + 1..].split(" ")
+        // Split, but only take the parts that do not denote options
+        // or flags.
+            .take_while(|p| p.chars().all(|c| c.is_alphabetic()))
+    } else {
+        // Odd...
+        return Ok(());
+    };
+
+    // Now traverse the CLI tree to find the subcommand.
+    let mut cmd = cli;
+    for sub in subcommands {
+        cmd = if let Some(c) = cmd.get_subcommands()
+            .find(|c| c.get_name() == sub)
+        {
+            c
+        } else {
+            // Very odd...
+            return Ok(());
+        };
+    }
+
+    // And print the examples, if any.
+    if let Some(examples) = cmd.get_after_help() {
+        eprintln!("\n{}", examples);
+    }
+
+    Ok(())
+}
+
+#[cfg(test)]
+mod test {
+    use super::*;
+
+    // Run some clap tests.
+    #[test]
+    fn verify_app() {
+        let cli = cli::build(true);
+        cli.debug_assert();
+    }
+}
