@@ -21,7 +21,6 @@ use sequoia_openpgp::{
         amalgamation::ValidAmalgamation,
         CertParser,
     },
-    crypto::hash::Digest,
     Fingerprint,
     packet::{Signature, UserID},
     parse::Parse,
@@ -183,7 +182,7 @@ pub fn verify(git: &Repository,
         policy_files.insert(policy_hash);
 
         // Scan for revoked certificates.
-        for authorization in policy.authorization.values() {
+        for authorization in policy.authorization().values() {
             for cert in CertParser::from_bytes(&authorization.keyring)? {
                 let cert = if let Ok(cert) = cert {
                     cert
@@ -217,9 +216,9 @@ pub fn verify(git: &Repository,
                        cert.fingerprint());
                     hard_revoked.insert(vc.fingerprint());
                     for k in vc.keys().subkeys().for_signing() {
-                        hard_revoked.insert(k.fingerprint());
+                        hard_revoked.insert(k.key().fingerprint());
                         t!("  Badlisting signing key {}",
-                           k.fingerprint());
+                           k.key().fingerprint());
                     }
 
                     continue;
@@ -228,9 +227,9 @@ pub fn verify(git: &Repository,
                 // Check if any of the signing keys are hard revoked.
                 for k in vc.keys().subkeys().for_signing() {
                     if is_hard_revoked(k.revocation_status()) {
-                        hard_revoked.insert(k.fingerprint());
+                        hard_revoked.insert(k.key().fingerprint());
                         t!("  Signing key {} hard revoked, bad listing",
-                           k.fingerprint());
+                           k.key().fingerprint());
                     }
                 }
             }
@@ -362,7 +361,7 @@ pub fn verify(git: &Repository,
             let parent_policy = Policy::parse_bytes(&parent_policy)?;
             let commit_policy = Policy::parse_bytes(read_policy(&commit_id)?)?;
 
-            commit_goodlist = commit_policy.commit_goodlist.clone();
+            commit_goodlist = commit_policy.commit_goodlist().clone();
 
             (parent_policy.verify(git, &commit_id, &commit_policy,
                                   &mut results.signer_keys,
@@ -383,7 +382,7 @@ pub fn verify(git: &Repository,
             } else {
                 commit_id.to_string()
             };
-            eprintln!("{}: Cached positive verification", id);
+            eprintln!("{}:\n  Cached positive verification", id);
         }
 
         match vresult {
@@ -518,7 +517,6 @@ pub fn verify(git: &Repository,
         assert_eq!(commit_info.children, 0);
         let authenticated_suffix = commit_info.authenticated_suffix;
         let traversed_middle = commit_info.traversed_middle;
-        drop(commit_info);
 
         for (parent_i, parent) in commit.parents().enumerate() {
             let parent_id = parent.id();
@@ -611,6 +609,18 @@ pub fn verify(git: &Repository,
     }
 
     if valid_path {
+        if trust_root == commit_range.0 {
+            eprintln!(
+                "Verified that there is an authenticated path from the trust root\n\
+                 {} to {}.",
+                trust_root, commit_range.1);
+        } else {
+            eprintln!(
+                "Verified that there is an authenticated path from the trust root\n\
+                 {} via {}\n\
+                 to {}.",
+                trust_root, commit_range.0, commit_range.1);
+        }
         Ok(())
     } else {
         if errors.is_empty() {
@@ -649,7 +659,7 @@ pub fn verify(git: &Repository,
 
 // Returns the SHA512 digest of the provided bytes.
 fn sha512sum(bytes: &[u8]) -> Result<Vec<u8>> {
-    let mut digest = HashAlgorithm::SHA512.context()?;
+    let mut digest = HashAlgorithm::SHA512.context()?.for_digest();
     digest.update(bytes);
 
     let mut key = Vec::with_capacity(32);
@@ -681,7 +691,7 @@ impl VerificationCache {
     }
 
     fn key(&self, policy: &[u8], commit: Oid) -> Result<persistent_set::Value> {
-        let mut digest = HashAlgorithm::SHA512.context()?;
+        let mut digest = HashAlgorithm::SHA512.context()?.for_digest();
         digest.update(policy);
         digest.update(commit.as_bytes());
 
