@@ -5,8 +5,9 @@ use openpgp::serialize::Serialize;
 
 mod common;
 use common::Environment;
+use common::Result;
 
-fn create_environment() -> anyhow::Result<(Environment, String)> {
+fn create_environment() -> Result<(Environment, String)> {
     Environment::scooby_gang_bootstrap()
 }
 
